package ru.myitschool.work.ui.login

import android.content.Context
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import dagger.hilt.android.lifecycle.HiltViewModel
import dagger.hilt.android.qualifiers.ApplicationContext
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.launch
import okhttp3.Credentials
import retrofit2.Retrofit
import retrofit2.converter.gson.GsonConverterFactory
import ru.myitschool.work.SessionManager
import ru.myitschool.work.api.ApiService
import ru.myitschool.work.api.UserAuthResponse
import ru.myitschool.work.core.Constants
import javax.inject.Inject

@HiltViewModel
class LoginViewModel @Inject constructor(
    @ApplicationContext private val context: Context,
) : ViewModel() {
    private val _state = MutableStateFlow(LoginState())
    val state = _state.asStateFlow()

    private val apiService: ApiService by lazy {
        Retrofit.Builder()
            .baseUrl(Constants.SERVER_ADDRESS)
            .addConverterFactory(GsonConverterFactory.create())
            .build()
            .create(ApiService::class.java)
    }

    fun authenticate(username: String, password: String) {
        if (isValidUsername(username)) {
            viewModelScope.launch {
                try {
                    val credentials = Credentials.basic(username, password) // Создаем Basic Auth заголовок
                    val response = apiService.authenticate(username, credentials) // Передаем заголовок в запрос
                    if (response.isSuccessful) {
                        val userAuthResponse = response.body() // Получаем ответ с ролью пользователя
                        userAuthResponse?.let {
                            SessionManager.userLogin = username // Сохраняем логин
                            SessionManager.userRole = it.role // Сохраняем роль
                        }
                        _state.value = LoginState(success = true, userAuthResponse = userAuthResponse)
                    } else if (response.code() == 503) { // Пример кода для техработ
                        _state.value = LoginState(maintenance = true)
                    } else {
                        _state.value = LoginState(error = "Ошибка авторизации")
                    }
                } catch (e: Exception) {
                    // Логирование ошибки
                    e.printStackTrace()
                    _state.value = LoginState(error = "Ошибка сети. Проверьте подключение к интернету.")
                }
            }
        } else {
            _state.value = LoginState(error = "Неправильный логин")
        }
    }

    private fun isValidUsername(username: String): Boolean {
        return username.length >= 3 && !username.first().isDigit() && username.all { it.isLetterOrDigit() }
    }

    data class LoginState(
        val success: Boolean = false,
        val error: String? = null,
        val maintenance: Boolean = false,
        val userAuthResponse: UserAuthResponse? = null // Добавляем поле для хранения информации о роли
    )
}